@@ -8,15 +8,17 @@ from typing import Any, Dict, List, Optional, Type, Union
 
 import backoff
 import humps
-from aiohttp import ClientResponse, ClientSession
+from aiohttp import ClientResponse, ClientSession, ServerDisconnectedError
 
 from pyhoma.exceptions import (
     BadCredentialsException,
+    InvalidCommandException,
+    MaintenanceException,
     NotAuthenticatedException,
     TooManyExecutionsException,
     TooManyRequestsException,
 )
-from pyhoma.models import Command, Device, Event, Execution, Scenario, State
+from pyhoma.models import Command, Device, Event, Execution, Gateway, Scenario, State
 
 JSON = Union[Dict[str, Any], List[Dict[str, Any]]]
 
@@ -51,7 +53,7 @@ class TahomaClient:
         self.api_url = api_url
 
         self.devices: List[Device] = []
-        self.__roles = None
+        self.gateways: List[Gateway] = []
         self.event_listener_id: Optional[str] = None
 
         self.session = session if session else ClientSession()
@@ -88,6 +90,12 @@ class TahomaClient:
             return True
         return False
 
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
     async def get_devices(self, refresh: bool = False) -> List[Device]:
         """
         List devices
@@ -101,6 +109,38 @@ class TahomaClient:
 
         return devices
 
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def get_gateways(self, refresh: bool = False) -> List[Gateway]:
+        """
+        List gateways
+        """
+        if self.gateways and not refresh:
+            return self.gateways
+
+        response = await self.__get("setup/gateways")
+        gateways = [Gateway(**g) for g in humps.decamelize(response)]
+        self.gateways = gateways
+
+        return gateways
+
+    @backoff.on_exception(
+        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+    )
+    async def get_device_definition(self, deviceurl: str) -> Optional[JSON]:
+        """
+        Retrieve a particular setup device definition
+        """
+        response = await self.__get(
+            f"setup/devices/{urllib.parse.quote_plus(deviceurl)}"
+        )
+
+        return response.get("definition")
+
     @backoff.on_exception(
         backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
     )
@@ -180,6 +220,7 @@ class TahomaClient:
 
         return executions
 
+    @backoff.on_exception(backoff.expo, TooManyExecutionsException, max_tries=10)
     @backoff.on_exception(
         backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
     )
@@ -219,7 +260,10 @@ class TahomaClient:
         return response["execId"]
 
     @backoff.on_exception(
-        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
     )
     async def get_scenarios(self) -> List[Scenario]:
         """ List the scenarios """
@@ -245,7 +289,7 @@ class TahomaClient:
     ) -> Any:
         """ Make a POST request to the TaHoma API """
         async with self.session.post(
-            f"{self.api_url}{endpoint}", data=data, json=payload
+            f"{self.api_url}{endpoint}", data=data, json=payload,
         ) as response:
             await self.check_response(response)
             return await response.json()
@@ -265,6 +309,8 @@ class TahomaClient:
             result = await response.json(content_type=None)
         except JSONDecodeError:
             result = await response.text()
+            if "Server is down for maintenance" in result:
+                raise MaintenanceException("Server is down for maintenance")
             raise Exception(
                 f"Unknown error while requesting {response.url}. {response.status} - {result}"
             )
@@ -289,4 +335,8 @@ class TahomaClient:
             if message == "Server busy, please try again later. (Too many executions)":
                 raise TooManyExecutionsException(message)
 
-            raise Exception(message if message else result)
+            # {"error": "UNSUPPORTED_OPERATION", "error": "No such command : ..."}
+            if "No such command" in message:
+                raise InvalidCommandException(message)
+
+        raise Exception(message if message else result)
